@@ -6,6 +6,9 @@ use zeroize::Zeroize;
 pub struct StoreConfig {
   pub name: String,
   pub store_url: String,
+  pub remote_url: Option<String>,
+  #[serde(default)]
+  pub sync_interval_sec: u32,
   pub client_id: String,
   pub autolock_timeout_secs: u64,
   pub default_identity_id: Option<String>,
