@@ -3,7 +3,10 @@ use crate::clipboard::SelectionProvider;
 use crate::otp::OTPAuthUrl;
 use log::{error, info};
 use std::time::{SystemTime, UNIX_EPOCH};
+use zeroize::{Zeroize, Zeroizing};
 
+#[derive(Clone, Zeroize)]
+#[zeroize(drop)]
 pub struct SecretsProvider {
   store_name: String,
   block_id: String,
@@ -13,7 +16,12 @@ pub struct SecretsProvider {
 
 impl SecretsProvider {
   pub fn new(store_name: String, block_id: String, secret_version: SecretVersion, properties: &[&str]) -> Self {
-    let properties_stack = properties.iter().rev().map(ToString::to_string).collect();
+    let properties_stack = properties
+      .iter()
+      .filter(|p| secret_version.properties.has_non_empty(p))
+      .rev()
+      .map(ToString::to_string)
+      .collect();
     SecretsProvider {
       store_name,
       block_id,
@@ -37,7 +45,7 @@ impl SelectionProvider for SecretsProvider {
       })
   }
 
-  fn get_selection_value(&self) -> Option<String> {
+  fn get_selection_value(&self) -> Option<Zeroizing<String>> {
     let property = self.properties_stack.last()?;
     let value = self.secret_version.properties.get(property)?;
 
@@ -46,7 +54,7 @@ impl SelectionProvider for SecretsProvider {
       match OTPAuthUrl::parse(value) {
         Ok(otpauth) => {
           let (token, _) = otpauth.generate(SystemTime::now().duration_since(UNIX_EPOCH).unwrap().as_secs());
-          Some(token)
+          Some(Zeroizing::new(token))
         }
         Err(error) => {
           error!("Invalid OTPAuth url: {}", error);
@@ -55,7 +63,7 @@ impl SelectionProvider for SecretsProvider {
       }
     } else {
       info!("Providing {} of {}", property, self.secret_version.secret_id);
-      Some(value.clone())
+      Some(Zeroizing::new(value.clone()))
     }
   }
 
